use log::trace;
use tokio::{sync::mpsc as tmpsc, task::JoinHandle};

mod error;
mod mpsc;
mod pub_sub;
mod registry;
mod request;

pub(crate) use error::{BrokerError, BrokerResult};
pub(crate) use mpsc::{
    ClaimReceiverError, ClaimReceiverRequest, SenderRequestNoWait, SenderRequestWait,
};
pub(crate) use pub_sub::{AdvertiseRequest, SubscribeRequest};
use registry::Registry;
pub(crate) use request::{BrokerRequestBox, BrokerRequestSender, MakeBrokerRequest};

pub(crate) struct BrokerTask {
    request_rx: tmpsc::Receiver<BrokerRequestBox>,
}

impl BrokerTask {
    pub(crate) fn start() -> (BrokerRequestSender, JoinHandle<()>) {
        let (request_tx, request_rx) = tmpsc::channel(8);

        let broker = BrokerTask { request_rx };
        let join_handle = tokio::spawn(broker.run());

        (request_tx, join_handle)
    }

    async fn run(mut self) {
        trace!("BrokerTask starting");

        let mut registry = Registry::default();

        loop {
            tokio::select! {
                Some(request) = self.request_rx.recv() => {
                    request.run_request(&mut registry);
                }
                else => {
                    trace!("no more handles to BrokerTask");
                    break;
                }
            }
        }

        trace!("BrokerTask exiting");
    }
}
