use super::{
    error::BrokerResult,
    registry::{BasicMessageType, Registry},
    request::{BrokerRequest, RequestHandler},
};
use crate::{pub_sub, Message};

pub type SubscribeRequest<T> = BrokerRequest<Subscribe<T>>;
pub struct Subscribe<T: Message> {
    msg_type: BasicMessageType<T>,
}

impl<T: Message> Default for Subscribe<T> {
    fn default() -> Self {
        Self {
            msg_type: BasicMessageType::<T>::default(),
        }
    }
}

impl<T: Message> RequestHandler for Subscribe<T> {
    type Response = BrokerResult<pub_sub::Subscription<T>>;

    fn handle(self, registry: &mut Registry) -> Self::Response {
        let sender = registry.get_broadcast_sender::<T>(&self.msg_type);
        sender.map(|sender| {
            let receiver = sender.subscribe();
            pub_sub::Subscription::new(receiver)
        })
    }
}

pub type AdvertiseRequest<T> = BrokerRequest<Advertise<T>>;
pub struct Advertise<T: Message> {
    msg_type: BasicMessageType<T>,
}

impl<T: Message> Default for Advertise<T> {
    fn default() -> Self {
        Self {
            msg_type: BasicMessageType::<T>::default(),
        }
    }
}

impl<T: Message> RequestHandler for Advertise<T> {
    type Response = BrokerResult<pub_sub::Publication<T>>;

    fn handle(self, registry: &mut Registry) -> Self::Response {
        let sender = registry.get_broadcast_sender::<T>(&self.msg_type);
        sender.map(|sender| pub_sub::Publication::new(sender.clone()))
    }
}
